/*
 * HSPMap.h
 *
 *  Created on: Mar 24, 2010
 *      Author: Oleksiy Kohany <kohany@gmail.com>
 */

#ifndef HSPMAP_H_
#define HSPMAP_H_

#include <list>
#include <map>
#include <string>
#include "HSP.h"
using namespace std;
namespace bio {

/*
 *
 */
class HSPMap {
public:
   typedef list<HSP> hsplist_t;
   typedef map<string, hsplist_t> hspmap_t;
   typedef hspmap_t::iterator iterator;
   typedef hspmap_t::const_iterator const_iterator;
   typedef hspmap_t::reverse_iterator reverse_iterator;
   typedef hspmap_t::const_reverse_iterator const_reverse_iterator;
private:
   hspmap_t _map;
   bool top_p, bot_p;
public:
   HSPMap(bool top_p = false, bool bot_p = false);
   virtual ~HSPMap();
   void sort();
   void write(ostream& os) const;
   HSPMap& read(istream& is, bool noSelf = false);
   hsplist_t& operator[](const string& name) {
      return _map[name];
   }
   iterator begin() {
      return _map.begin();
   }
   iterator end() {
      return _map.end();
   }
   reverse_iterator rbegin() {
      return _map.rbegin();
   }
   reverse_iterator rend() {
      return _map.rend();
   }
   const_iterator begin() const {
      return _map.begin();
   }
   const_iterator end() const {
      return _map.end();
   }
   const_reverse_iterator rbegin() const {
      return _map.rbegin();
   }
   const_reverse_iterator rend() const {
      return _map.rend();
   }
   size_t size() const {
      return _map.size();
   }
   iterator find ( const hspmap_t::key_type& x ) {
      return _map.find(x);
   }
};

istream& operator >>(istream& is, HSPMap& mp);
ostream& operator <<(ostream& os, const HSPMap& mp);
};
#endif /* HSPMAP_H_ */
