/***************************************************************************
 main.cpp  -  description
 -------------------
 begin                : Wed Jan  8 11:03:45 UTC 2003
 copyright            : (C) 2003 by Oleksiy Kohany
 email                : kohany@girinst.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <string.h>
#include "HSP.h"
#include "HSPMap.h"
#include "functions.h"

using namespace std;
using namespace bio;

/**
 * Masks sequences read from istream& according to fragments' position
 * in hspmap. If mask_lc==true makes those regions lowercase, otherwise
 * replaces all symbols with mask_char.
 * is and os can point to the same iostream - should be no problem as long
 * as getline(is, line) removes a single '\n' symbol from the end of line.
 * */
void maskMap(HSPMap& hspmap, istream& is, ostream& os, bool mask_lc = false, char mask_char = 'N') {
   size_t pos = 0;
   char c;
   string line;
   string name;
   istringstream iss;
   HSPMap::iterator mptr;
   HSPMap::hsplist_t::iterator lptr;

   while (getline(is, line)) {
      switch (line[0]) {
      case '>':
         iss.clear();
         iss.str(line.substr(1));
         iss >> name;
         os << line << "\n";
         break;
      case ';':
         do {
            os << line << "\n";
         } while (getline(is, line) && line[0] == ';');
         iss.clear();
         iss.str(line);
         iss >> name;
         os << line << "\n";
         break;
      default:
         if (name.size() > 0) {
            if (mptr != hspmap.end() && lptr != (*mptr).second.end()) {
               for (string::iterator sptr = line.begin(); sptr != line.end(); sptr++) {
                  if (isgraph(*sptr)) {
                     while (lptr != (*mptr).second.end() && (*lptr).getTop().getEnd() <= pos) {
                        lptr++;
                     }
                     if (lptr != (*mptr).second.end() && (*lptr).getTop().getBeg() <= pos) {
                        *sptr = mask_lc ? tolower(*sptr) : mask_char;
                     }
                     pos++;
                  }
               }
            }
         } else {
            throw invalid_argument("Did not find expected '>' or ';'");
         }
         os << line << "\n";
         continue;
      }
      mptr = hspmap.find(name);
      if (mptr != hspmap.end()) {
         lptr = (*mptr).second.begin();
         pos = 0;
      }
   }
}
//====================================================================

void usage(const char* exec) {
   if (!exec) {
      return;
   }
   if (strrchr(exec, '/')) {
      exec = strrchr(exec, '/') + 1;
   }
   cerr << "NAME\n\t" << exec;
#ifdef VERSION
   cerr << " V" << VERSION;
#endif
   cerr << " - masks out fragments from genomic sequences\n";
   cerr << "\nSYNOPSIS\n\t" << exec << " [inmap] infile [outfile] [-c MASKCHAR]\n";
   cerr << "\nDESCRIPTION\n\tMasks out fragments from genomic sequences.\n";
   cerr << "\tMasked sequences will be saved in outfile.\n";
   cerr << "OPTIONS\n\t-c\n\t   sets masking symbol. Default is 'N'\n";
   cerr << "FLAGS\n\t-lc\n\t   activates lowercase masking\n";
#ifdef COPYRIGHT
   cerr << "\nCOPYRIGHT\n\t" << COPYRIGHT << "\n";
#endif
}
//====================================================================
int main(int argc, char *argv[]) {
   istream *mfs;
   ifstream *ifs;
   ostream *ofs;

   char **args = argv;
   char *outname = 0;
   ARGMAP amap;
   bool bLC = 0;
   char mask_char = 'N';
   getoptions(argv, amap, 1, "-c");
   getflags(argv, amap, 3, "-lc", "--help", "--version");
   if (amap.find("-c") != amap.end()) {
      sscanf(amap["-c"].c_str(), "%c", &mask_char);
   }
   if (amap.find("--help") != amap.end() || amap.find("--version") != amap.end()) {
      usage(argv[0]);
      exit(0);
   }
   if (amap.find("-lc") != amap.end()) {
      bLC = 1;
   }
   if ((args = assigninput(mfs, args)) == 0) {
      usage(argv[0]);
      exit(10);
   }
   if (args[1] == 0) {
      usage(argv[0]);
      exit(20);
   }
   if ((ifs = openifs(args[1])) == 0) {
      cerr << "Failed to open " << args[1] << "\n";
      usage(argv[0]);
      exit(30);
   }
   args++;
   if ((args = assignoutput(ofs, args)) == 0) {
      usage(argv[0]);
      exit(50);
   }
   //===================================================================
   HSPMap hspmap;
   if (hspmap.read(*mfs).size() > 0) {
      maskMap(hspmap, *ifs, *ofs, bLC, mask_char);
   }
   //===================================================================
   delete ifs;
   if(mfs != &cin) {
      delete mfs;
   }
   if (ofs != &cout) {
      delete ofs;
   }
   return EXIT_SUCCESS;
}
